# chosung.py

from hangulpy.utils import CHOSUNG_LIST, CHOSUNG_BASE, is_hangul_char, HANGUL_BEGIN_UNICODE

def get_chosung_string(text, keep_spaces=False):
	"""
	주어진 문자열의 각 문자의 초성을 반환합니다.

	:param text: 한글 문자열
	:param keep_spaces: 공백을 유지할지 여부 (기본값: False)
	:return: 초성 문자열
	"""
	def extract_chosung(c):
		if is_hangul_char(c):
			char_index = ord(c) - HANGUL_BEGIN_UNICODE
			chosung_index = char_index // CHOSUNG_BASE
			return CHOSUNG_LIST[chosung_index]
		return c

	if keep_spaces:
		return ''.join(extract_chosung(c) if is_hangul_char(c) else c for c in text)
	else:
		return ''.join(extract_chosung(c) for c in text if is_hangul_char(c) or not c.isspace())

def chosungIncludes(word, pattern):
	"""
	주어진 단어에 패턴의 초성이 포함되어 있는지 확인합니다.

	:param word: 검색할 단어
	:param pattern: 검색할 초성 패턴
	:return: 패턴이 단어의 초성에 포함되어 있으면 True, 아니면 False
	"""
	# 단어의 각 문자를 초성으로 변환하여 문자열을 생성합니다.
	def extract_chosung(c):
		if is_hangul_char(c):
			char_index = ord(c) - HANGUL_BEGIN_UNICODE
			chosung_index = char_index // CHOSUNG_BASE
			return CHOSUNG_LIST[chosung_index]
		return c

	word_chosung = ''.join(extract_chosung(c) for c in word)
	# 패턴이 초성 문자열에 포함되어 있는지 확인합니다.
	return pattern in word_chosung



# noun.py

from hangulpy.utils import is_hangul_char, HANGUL_BEGIN_UNICODE, JONGSUNG_COUNT
from hangulpy.josa import has_jongsung

def jarip_noun(word, particle):
    """
    주어진 단어에 적절한 자립명사를 붙여 반환합니다.

    :param word: 자립명사와 결합할 단어
    :param particle: 붙일 자립명사 ('율/률', '열/렬', '영/령', '염/념', '예/례')
    :return: 적절한 자립명사가 붙은 단어 문자열
    """
    if not word:
        return ''

    # 단어의 마지막 글자를 가져옵니다.
    word_ending = word[-1]

    # 마지막 글자의 받침 유무를 확인합니다.
    jongsung_exists = has_jongsung(word_ending)

    if particle == '율/률':
        return word + ('율' if not jongsung_exists else '률')
    elif particle == '열/렬':
        return word + ('열' if not jongsung_exists else '렬')
    elif particle == '영/령':
        return word + ('영' if not jongsung_exists else '령')
    elif particle == '염/념':
        return word + ('염' if not jongsung_exists else '념')
    elif particle == '예/례':
        return word + ('예' if not jongsung_exists else '례')
    else:
        raise ValueError(f"Unsupported particle: {particle}")



# hangul_split.py

from hangulpy.utils import CHOSUNG_LIST, CHOSUNG_BASE, is_hangul_char, HANGUL_BEGIN_UNICODE, JUNGSUNG_LIST, JUNGSUNG_DECOMPOSE, JONGSUNG_LIST, JONGSUNG_DECOMPOSE, JUNGSUNG_BASE

def split_hangul_char(c):
	"""
	주어진 한글 음절을 초성, 중성, 종성으로 분해하여 배열 형태로 반환합니다.
	
	:param c: 한글 음절 문자
	:return: 초성, 중성, 종성을 배열 형태로 반환
	"""
	if is_hangul_char(c):
		# 한글 음절의 유니코드 값을 기준으로 각 성분의 인덱스를 계산합니다.
		char_index = ord(c) - HANGUL_BEGIN_UNICODE
		chosung_index = char_index // CHOSUNG_BASE
		jungsung_index = (char_index % CHOSUNG_BASE) // JUNGSUNG_BASE
		jongsung_index = char_index % JUNGSUNG_BASE

		# 중성 및 종성 분해
		jungsung = JUNGSUNG_LIST[jungsung_index]
		jongsung = JONGSUNG_LIST[jongsung_index]

		jungsung_decomposed = JUNGSUNG_DECOMPOSE.get(jungsung, [jungsung])
		jongsung_decomposed = JONGSUNG_DECOMPOSE.get(jongsung, [jongsung])

		return [CHOSUNG_LIST[chosung_index]] + list(jungsung_decomposed) + list(jongsung_decomposed)
	
	# 자모인 경우 추가 처리
	if c in JUNGSUNG_DECOMPOSE:
		return list(JUNGSUNG_DECOMPOSE[c])
	if c in JONGSUNG_DECOMPOSE:
		return list(JONGSUNG_DECOMPOSE[c])

	return [c]  # 한글이 아니면 그대로 반환

def split_hangul_string(s):
	"""
	주어진 문자열의 각 한글 음절을 초성, 중성, 종성으로 분해하여 배열 형태로 반환합니다.
	
	:param s: 문자열
	:return: 각 한글 음절을 초성, 중성, 종성으로 분해한 결과를 포함하는 배열
	"""
	result = []
	for char in s:
		result.extend(split_hangul_char(char))
	return result


# hangul_role.py

from hangulpy.utils import CHOSUNG_LIST, JONGSUNG_LIST

def can_be_chosung(char):
	"""
	주어진 문자가 한글 초성으로 쓰일 수 있는지 확인합니다.
	
	:param char: 확인할 문자
	:return: 초성으로 쓰일 수 있으면 True, 아니면 False
	"""
	return char in CHOSUNG_LIST

def can_be_jongsung(char):
	"""
	주어진 문자가 한글 종성으로 쓰일 수 있는지 확인합니다.
	
	:param char: 확인할 문자
	:return: 종성으로 쓰일 수 있으면 True, 아니면 False
	"""
	return char in JONGSUNG_LIST



# utils.py

# 유니코드 한글 시작과 끝
# 유니코드에서 한글 음절은 U+AC00에서 U+D7A3까지 할당되어 있습니다.
HANGUL_BEGIN_UNICODE = 0xAC00  # '가'의 유니코드 값
HANGUL_END_UNICODE = 0xD7A3	# '힣'의 유니코드 값

# 초성 리스트
# 한글의 초성 19개를 정의합니다.
CHOSUNG_LIST = [
	'ㄱ', 'ㄲ', 'ㄴ', 'ㄷ', 'ㄸ', 'ㄹ', 'ㅁ', 'ㅂ', 'ㅃ', 'ㅅ', 'ㅆ', 'ㅇ', 'ㅈ', 'ㅉ', 'ㅊ', 'ㅋ', 'ㅌ', 'ㅍ', 'ㅎ'
]

# 초성 인덱스 계산 상수
# 초성, 중성, 종성의 조합으로 한글 음절을 만들 때, 초성의 위치를 계산하는 데 사용합니다.
CHOSUNG_BASE = 588  # 초성당 588개의 음절이 있습니다. (21 * 28)

# 중성 리스트
# 한글의 중성 21개를 정의합니다.
JUNGSUNG_LIST = [
	'ㅏ', 'ㅐ', 'ㅑ', 'ㅒ', 'ㅓ', 'ㅔ', 'ㅕ', 'ㅖ', 'ㅗ', 'ㅘ', 'ㅙ', 'ㅚ', 'ㅛ', 'ㅜ', 'ㅝ', 'ㅞ', 'ㅟ', 'ㅠ', 'ㅡ', 'ㅢ', 'ㅣ'
]

# 복합 중성 분해 정보
JUNGSUNG_DECOMPOSE = {
	'ㅘ': ('ㅗ', 'ㅏ'),
	'ㅙ': ('ㅗ', 'ㅐ'),
	'ㅚ': ('ㅗ', 'ㅣ'),
	'ㅝ': ('ㅜ', 'ㅓ'),
	'ㅞ': ('ㅜ', 'ㅔ'),
	'ㅟ': ('ㅜ', 'ㅣ'),
	'ㅢ': ('ㅡ', 'ㅣ')
}

# 종성 리스트
# 한글의 종성 28개를 정의합니다.
JONGSUNG_LIST = [
	'', 'ㄱ', 'ㄲ', 'ㄳ', 'ㄴ', 'ㄵ', 'ㄶ', 'ㄷ', 'ㄹ', 'ㄺ', 'ㄻ', 'ㄼ', 'ㄽ', 'ㄾ', 'ㄿ', 'ㅀ', 'ㅁ', 'ㅂ', 'ㅄ', 'ㅅ', 'ㅆ', 'ㅇ', 'ㅈ', 'ㅊ', 'ㅋ', 'ㅌ', 'ㅍ', 'ㅎ'
]

# 복합 종성 분해 정보
JONGSUNG_DECOMPOSE = {
	'ㄳ': ('ㄱ', 'ㅅ'),
	'ㄵ': ('ㄴ', 'ㅈ'),
	'ㄶ': ('ㄴ', 'ㅎ'),
	'ㄺ': ('ㄹ', 'ㄱ'),
	'ㄻ': ('ㄹ', 'ㅁ'),
	'ㄼ': ('ㄹ', 'ㅂ'),
	'ㄽ': ('ㄹ', 'ㅅ'),
	'ㄾ': ('ㄹ', 'ㅌ'),
	'ㄿ': ('ㄹ', 'ㅍ'),
	'ㅀ': ('ㄹ', 'ㅎ'),
	'ㅄ': ('ㅂ', 'ㅅ')
}

# 중성 인덱스 계산 상수
# 중성의 위치를 계산하는 데 사용합니다.
JUNGSUNG_BASE = 28  # 중성당 28개의 음절이 있습니다.

# 받침 유무를 확인하기 위한 상수
# 한글 음절의 받침 개수는 28개입니다.
JONGSUNG_COUNT = 28

# 숫자 읽기 관련 코드
UNITS = [
	"", "십", "백", "천", "만", "십만", "백만", "천만", "억", "십억", "백억", "천억",
	"조", "십조", "백조", "천조", "경", "십경", "백경", "천경", "해", "십해", "백해", "천해",
	"자", "십자", "백자", "천자", "양", "십양", "백양", "천양", "구", "십구", "백구", "천구",
	"간", "십간", "백간", "천간", "정", "십정", "백정", "천정", "재", "십재", "백재", "천재",
	"극", "십극", "백극", "천극", "항하사", "십항하사", "백항하사", "천항하사", "아승기", "십아승기", "백아승기", "천아승기",
	"나유타", "십나유타", "백나유타", "천나유타", "불가사의", "십불가사의", "백불가사의", "천불가사의", "무량대수", "십무량대수", "백무량대수", "천무량대수",
	"겁", "십겁", "백겁", "천겁", "훈공", "십훈공", "백훈공", "천훈공"
]

NUMBERS = ["", "일", "이", "삼", "사", "오", "육", "칠", "팔", "구"]

def is_hangul_char(c):
	"""
	주어진 문자가 한글 음절인지 확인합니다.
	
	:param c: 확인할 문자
	:return: 한글 음절이면 True, 아니면 False
	"""
	return HANGUL_BEGIN_UNICODE <= ord(c) <= HANGUL_END_UNICODE



# hangul_check.py

from hangulpy.utils import CHOSUNG_LIST, JUNGSUNG_LIST

def is_hangul_consonant(char):
	"""
	주어진 문자가 한글 자음인지 확인합니다.
	
	:param char: 확인할 문자
	:return: 한글 자음이면 True, 아니면 False
	"""
	return char in CHOSUNG_LIST

def is_hangul_vowel(char):
	"""
	주어진 문자가 한글 모음인지 확인합니다.
	
	:param char: 확인할 문자
	:return: 한글 모음이면 True, 아니면 False
	"""
	return char in JUNGSUNG_LIST



# hangul_contains.py

from hangulpy.utils import is_hangul_char
from hangulpy.hangul_split import split_hangul_char

def hangul_contains(word, pattern, notallowempty=False):
	"""
	주어진 한글 문자열이 다른 한글 문자열을 포함하는지 검사합니다.
	
	:param word: 검사할 한글 문자열
	:param pattern: 포함 여부를 검사할 한글 문자열 패턴
	:param notallowempty: 패턴이 빈 문자열일 때 false를 반환하는 옵션
	:return: 포함되면 True, 아니면 False
	"""
	if not pattern:
		return not notallowempty
	
	# 문자열을 분해하여 리스트로 변환
	word_split = ''.join(''.join(split_hangul_char(char)) for char in word)
	pattern_split = ''.join(''.join(split_hangul_char(char)) for char in pattern)
	
	return pattern_split in word_split



# hangul_number.py

from hangulpy.utils import UNITS, NUMBERS

def number_to_hangul(num):
	"""
	주어진 숫자를 한글로 변환합니다.
	
	:param num: 변환할 숫자
	:return: 한글 숫자 문자열
	"""
	if num == 0:
		return "영"
	
	if isinstance(num, float):
		return float_to_hangul(num)
	
	result = ""
	str_num = str(num)[::-1]
	
	for i, digit in enumerate(str_num):
		digit = int(digit)
		if digit != 0:
			if i == 4:
				result = "만" + result
			elif i > 4 and i % 4 == 0:
				result = UNITS[4 * (i // 4)] + result
			part = NUMBERS[digit] + UNITS[i % 4]
			if i % 4 == 0 and digit == 1 and i != 0:
				part = UNITS[i % 4]  # 1은 생략
			result = part + result

	if result.startswith('일'):
		result = result[1:]
	
	return result

def float_to_hangul(num):
	"""
	주어진 소수를 한글로 변환합니다.
	
	:param num: 변환할 소수
	:return: 한글 숫자 문자열
	"""
	int_part, frac_part = str(num).split('.')
	return f"{number_to_hangul(int(int_part))} 점 {''.join(NUMBERS[int(d)] for d in frac_part)}"

def hangul_to_number(hangul):
	"""
	주어진 한글 숫자 문자열을 숫자로 변환합니다.
	
	:param hangul: 변환할 한글 숫자 문자열
	:return: 숫자
	"""
	num_map = {name: value for value, name in enumerate(NUMBERS) if name}
	unit_map = {name: 10**idx for idx, name in enumerate(UNITS) if name}
	num = 0
	tmp = 0
	stack = []
	is_float = False
	float_part = 0
	float_div = 1

	# 소수점 분리
	if '점' in hangul:
		integer_part, fractional_part = hangul.split('점')
	else:
		integer_part, fractional_part = hangul, ''

	# 정수 부분 처리
	for char in integer_part:
		if char in num_map:
			stack.append(num_map[char])
		elif char in unit_map:
			if not stack:
				stack.append(1)
			tmp += sum(stack) * unit_map[char]
			stack = []
		else:
			if stack:
				tmp += sum(stack)
			stack = []
			num += tmp
			tmp = 0

	if stack:
		tmp += sum(stack)
	num += tmp

	# 소수 부분 처리
	if fractional_part:
		for char in fractional_part:
			if char in num_map:
				float_part = float_part * 10 + num_map[char]
				float_div *= 10
		return num + float_part / float_div

	return num



# josa.py

from hangulpy.utils import is_hangul_char, HANGUL_BEGIN_UNICODE, JONGSUNG_COUNT

def has_jongsung(char):
	"""
	주어진 한글 음절에 받침이 있는지 확인합니다.
	
	:param char: 한글 음절 문자
	:return: 받침이 있으면 True, 없으면 False
	"""
	if is_hangul_char(char):
		# 한글 음절의 유니코드 값을 기준으로 받침의 유무를 확인합니다.
		char_index = ord(char) - HANGUL_BEGIN_UNICODE
		return (char_index % JONGSUNG_COUNT) != 0
	return False

def josa(word, particle):
	"""
	주어진 단어에 적절한 조사를 붙여 반환합니다.
	
	:param word: 조사와 결합할 단어
	:param particle: 붙일 조사 ('을/를', '이/가', '은/는', '와/과', '으로/로', '이나/나', '이에/에', '이란/란', '아/야', '이랑/랑', '이에요/예요', '으로서/로서', '으로써/로써', '으로부터/로부터', '이여/여', '께서', '이야/야', '와서/와', '이라서/라서', '이든/든', '이며/며', '이라도/라도', '이니까/니까', '이지만/지만', '이랑은/랑은'
								'이라고/라고', '이라며/라며', '이라니/라니', '이라니까/라니까', '이라거든/라거든', '이라더니/라더니', '이라더군/라더군', '이라던데/라던데', '이라고는/라고는', '이라는데/라는데', '이라면/라면', '이라서야/라서야', '이라야/라야', '이라든가/라든가', '이든지/든지', '이거나/거나', '이라면야/라면야', '이라면말이지/라면말이지',
								'이라야만/라야만', '이었으면/였으면', '이라서도/라서도', '이므로/므로', '이기에/기에', '이니/니', '이라니깐/라니깐', '이면서/면서', '이자/자', '이면서도/면서도', '이라든지/라든지', '이었지만/였지만', '이었으나/였으나', '이긴 하지만/긴 하지만', '이야말로/야말로', '이어야/여야', '이었고/였고', '이었는데/였는데', '이었더니/였더니',
								'이었을 때/였을 때', '이었을지라도/였을지라도', '이었던/였던', '이었으니까/였으니까', '이라고도/라고도', '이라곤 해도/라곤 해도', '이라지/라지', '이라네/라네', '이거든/거든', '이여/여', '이시여/시여', '아/야', '이야/야', '이에요/예요', '이어요/여요', '이었어요/였어요', '이었어/였어')
	:return: 적절한 조사가 붙은 단어 문자열
	"""
	if not word:
		return ''
	
	# 단어의 마지막 글자를 가져옵니다.
	word_ending = word[-1]
	# 마지막 글자의 받침 유무를 확인합니다.
	jongsung_exists = has_jongsung(word_ending)
	
	if particle == '을/를':
		return word + ('을' if jongsung_exists else '를')
	elif particle == '이/가':
		return word + ('이' if jongsung_exists else '가')
	elif particle == '은/는':
		return word + ('은' if jongsung_exists else '는')
	elif particle == '와/과':
		return word + ('과' if jongsung_exists else '와')
	elif particle == '으로/로':
		return word + ('으로' if jongsung_exists else '로')
	elif particle == '이나/나':
		return word + ('이나' if jongsung_exists else '나')
	elif particle == '이에/에':
		return word + ('이에' if jongsung_exists else '에')
	elif particle == '이란/란':
		return word + ('이란' if jongsung_exists else '란')
	elif particle == '아/야':
		return word + ('아' if jongsung_exists else '야')
	elif particle == '이랑/랑':
		return word + ('이랑' if jongsung_exists else '랑')
	elif particle == '이에요/예요':
		return word + ('이에요' if jongsung_exists else '예요')
	elif particle == '으로서/로서':
		return word + ('으로서' if jongsung_exists else '로서')
	elif particle == '으로써/로써':
		return word + ('으로써' if jongsung_exists else '로써')
	elif particle == '으로부터/로부터':
		return word + ('으로부터' if jongsung_exists else '로부터')
	elif particle == '이여/여':
		return word + ('이여' if jongsung_exists else '여')
	elif particle == '이야/야':
		return word + ('이야' if jongsung_exists else '야')
	elif particle == '와서/와':
		return word + ('와서' if jongsung_exists else '와')
	elif particle == '이라서/라서':
		return word + ('이라서' if jongsung_exists else '라서')
	elif particle == '이든/든':
		return word + ('이든' if jongsung_exists else '든')
	elif particle == '이며/며':
		return word + ('이며' if jongsung_exists else '며')
	elif particle == '이라도/라도':
		return word + ('이라도' if jongsung_exists else '라도')
	elif particle == '이니까/니까':
		return word + ('이니까' if jongsung_exists else '니까')
	elif particle == '이지만/지만':
		return word + ('이지만' if jongsung_exists else '지만')
	elif particle == '이랑은/랑은':
		return word + ('이랑은' if jongsung_exists else '랑은')
	elif particle == '이라고/라고':
		return word + ('이라고' if jongsung_exists else '라고')
	elif particle == '이라며/라며':
		return word + ('이라며' if jongsung_exists else '라며')
	elif particle == '이라니/라니':
		return word + ('이라니' if jongsung_exists else '라니')
	elif particle == '이라니까/라니까':
		return word + ('이라니까' if jongsung_exists else '라니까')
	elif particle == '이라거든/라거든':
		return word + ('이라거든' if jongsung_exists else '라거든')
	elif particle == '이라더니/라더니':
		return word + ('이라더니' if jongsung_exists else '라더니')
	elif particle == '이라더군/라더군':
		return word + ('이라더군' if jongsung_exists else '라더군')
	elif particle == '이라던데/라던데':
		return word + ('이라던데' if jongsung_exists else '라던데')
	elif particle == '이라고는/라고는':
		return word + ('이라고는' if jongsung_exists else '라고는')
	elif particle == '이라는데/라는데':
		return word + ('이라는데' if jongsung_exists else '라는데')
	elif particle == '이라면/라면':
		return word + ('이라면' if jongsung_exists else '라면')
	elif particle == '이라서야/라서야':
		return word + ('이라서야' if jongsung_exists else '라서야')
	elif particle == '이라야/라야':
		return word + ('이라야' if jongsung_exists else '라야')
	elif particle == '이라든가/라든가':
		return word + ('이라든가' if jongsung_exists else '라든가')
	elif particle == '이든지/든지':
		return word + ('이든지' if jongsung_exists else '든지')
	elif particle == '이거나/거나':
		return word + ('이거나' if jongsung_exists else '거나')
	elif particle == '이라면야/라면야':
		return word + ('이라면야' if jongsung_exists else '라면야')
	elif particle == '이라면말이지/라면말이지':
		return word + ('이라면말이지' if jongsung_exists else '라면말이지')
	elif particle == '이라야만/라야만':
		return word + ('이라야만' if jongsung_exists else '라야만')
	elif particle == '이었으면/였으면':
		return word + ('이었으면' if jongsung_exists else '였으면')
	elif particle == '이라서도/라서도':
		return word + ('이라서도' if jongsung_exists else '라서도')
	elif particle == '이므로/므로':
		return word + ('이므로' if jongsung_exists else '므로')
	elif particle == '이기에/기에':
		return word + ('이기에' if jongsung_exists else '기에')
	elif particle == '이니/니':
		return word + ('이니' if jongsung_exists else '니')
	elif particle == '이라니깐/라니깐':
		return word + ('이라니깐' if jongsung_exists else '라니깐')
	elif particle == '이면서/면서':
		return word + ('이면서' if jongsung_exists else '면서')
	elif particle == '이자/자':
		return word + ('이자' if jongsung_exists else '자')
	elif particle == '이면서도/면서도':
		return word + ('이면서도' if jongsung_exists else '면서도')
	elif particle == '이라든지/라든지':
		return word + ('이라든지' if jongsung_exists else '라든지')
	elif particle == '이었지만/였지만':
		return word + ('이었지만' if jongsung_exists else '였지만')
	elif particle == '이었으나/였으나':
		return word + ('이었으나' if jongsung_exists else '였으나')
	elif particle == '이긴 하지만/긴 하지만':
		return word + ('이긴 하지만' if jongsung_exists else '긴 하지만')
	elif particle == '이야말로/야말로':
		return word + ('이야말로' if jongsung_exists else '야말로')
	elif particle == '이어야/여야':
		return word + ('이어야' if jongsung_exists else '여야')
	elif particle == '이었고/였고':
		return word + ('이었고' if jongsung_exists else '였고')
	elif particle == '이었는데/였는데':
		return word + ('이었는데' if jongsung_exists else '였는데')
	elif particle == '이었더니/였더니':
		return word + ('이었더니' if jongsung_exists else '였더니')
	elif particle == '이었을 때/였을 때':
		return word + ('이었을 때' if jongsung_exists else '였을 때')
	elif particle == '이었을지라도/였을지라도':
		return word + ('이었을지라도' if jongsung_exists else '였을지라도')
	elif particle == '이었던/였던':
		return word + ('이었던' if jongsung_exists else '였던')
	elif particle == '이었으니까/였으니까':
		return word + ('이었으니까' if jongsung_exists else '였으니까')
	elif particle == '이라고도/라고도':
		return word + ('이라고도' if jongsung_exists else '라고도')
	elif particle == '이라곤 해도/라곤 해도':
		return word + ('이라곤 해도' if jongsung_exists else '라곤 해도')
	elif particle == '이라지/라지':
		return word + ('이라지' if jongsung_exists else '라지')
	elif particle == '이라네/라네':
		return word + ('이라네' if jongsung_exists else '라네')
	elif particle == '이거든/거든':
		return word + ('이거든' if jongsung_exists else '거든')
	elif particle == '이시여/시여':
		return word + ('이시여' if jongsung_exists else '시여')
	elif particle == '이어요/여요':
		return word + ('이어요' if jongsung_exists else '여요')
	elif particle == '이었어요/였어요':
		return word + ('이었어요' if jongsung_exists else '였어요')
	elif particle == '이었어/였어':
		return word + ('이었어' if jongsung_exists else '였어')
	else:
		raise ValueError(f"Unsupported particle: {particle}")



from setuptools import setup, find_packages

setup(
	name='hangulpy',
	version='1.1.0',
	description='A Python library for processing Hangul, inspired by es-hangul.',
	long_description=open('README.md', encoding='utf-8').read(),
	long_description_content_type='text/markdown',
	url='https://github.com/gaon12/hangulpy',
	author='Jeong Gaon',
	author_email='gokirito12@gmail.com',
	license='MIT',
	packages=find_packages(),
	install_requires=[],
	classifiers=[
		'Programming Language :: Python :: 3',
		'License :: OSI Approved :: MIT License',
		'Operating System :: OS Independent',
	],
	python_requires='>=3.8',
)



# hangul_decompose.py

from hangulpy.utils import CHOSUNG_LIST, CHOSUNG_BASE, is_hangul_char, HANGUL_BEGIN_UNICODE, JUNGSUNG_LIST, JUNGSUNG_DECOMPOSE, JONGSUNG_LIST, JONGSUNG_DECOMPOSE, JUNGSUNG_BASE

def decompose_hangul_char(c):
	"""
	주어진 한글 음절을 초성, 중성, 종성으로 분해합니다.
	
	:param c: 한글 음절 문자
	:return: (초성, 중성, 종성) 튜플
	"""
	if is_hangul_char(c):
		# 한글 음절의 유니코드 값을 기준으로 각 성분의 인덱스를 계산합니다.
		char_index = ord(c) - HANGUL_BEGIN_UNICODE
		chosung_index = char_index // CHOSUNG_BASE
		jungsung_index = (char_index % CHOSUNG_BASE) // JUNGSUNG_BASE
		jongsung_index = char_index % JUNGSUNG_BASE
		
		# 중성 및 종성 분해
		jungsung = JUNGSUNG_LIST[jungsung_index]
		jongsung = JONGSUNG_LIST[jongsung_index]

		jungsung_decomposed = JUNGSUNG_DECOMPOSE.get(jungsung, (jungsung,))
		jongsung_decomposed = JONGSUNG_DECOMPOSE.get(jongsung, (jongsung,))
		
		return (CHOSUNG_LIST[chosung_index], jungsung_decomposed, jongsung_decomposed)
	
	return (c, '', '')  # 한글이 아니면 그대로 반환
